# New BSD License
# ===============
# Copyright (c) 2007, Graham Binns http://launchpad.net/~gmb
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in
#       the documentation and/or other materials provided with the
#       distribution.
#
#     * Neither the name of the <ORGANIZATION> nor the names of its
#       contributors may be used to endorse or promote products derived
#       from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""Clog Actions which use remote XML-RPC services."""

__metaclass__ = type

import os
import tempfile

from optparse import make_option

from cloglib.actions.base import register_action, Action


class XMLRPCAction(Action):
    """An action that connects to an XMLRPC server."""

    config_file = 'sites.conf'

    def __init__(self, arguments):
        super(XMLRPCAction, self).__init__(arguments)

    def initialise(self):
        """See `Action`."""
        # The first argument we get, if any, will be the section name
        # for the XMLRPC service in sites.conf.
        if len(self.args) > 0:
            if not self.config.has_section(self.args[0]):
                raise NoSuchSiteError(self.args[0])
            else:
                self.site = self.args[0]
        elif self.config.has_option('DEFAULT', 'default_site'):
            self.site = self.config.get('DEFAULT', 'default_site')
        else:
            self.site = None

        self.server_proxy = MetaWeblog(
            url=self.xmlrpc_url,
            username=self.username,
            password=self.password,
            blog=1)

    @property
    def xmlrpc_url(self):
        """Return the URL of the remote XMLRPC service.

        If an XMLRPC URL has been specified at the command line this
        will take precedent over any xmlrpc_url options in the config
        file.
        """
        if self.options.xmlrpc is not None:
            return self.options.xmlrpc

        # If we have been given a site to use, we try looking that up in
        # the config.
        if self.config.has_section(self.site):
            return self.config.get(self.site, 'xmlrpc_url')
        else:
            return None

    @property
    def username(self):
        """Return the username to be used with the remote XMLRPC service.

        If a username has been specified at the command line this will
        take precedent over any username options in the config file.
        """
        if self.options.user is not None:
            return self.options.user

        # If we have been given a site to use, we try looking that up in
        # the config.
        if self.config.has_section(self.site):
            return self.config.get(self.site, 'username')
        else:
            return None

    @property
    def password(self):
        """Return the password to be used with the remote XMLRPC service.

        If a password has been specified at the command line this will
        take precedent over any password options in the config file.
        """
        if self.options.password is not None:
            return self.options.password

        # If we have been given a site to use, we try looking that up in
        # the config.
        if self.config.has_section(self.site):
            return self.config.get(self.site, 'password')
        else:
            return None

    def getOptions(self):
        """See `Action`."""
        options = super(XMLRPCAction, self).getOptions() + [
            make_option('-x', '--xmlrpc', dest='xmlrpc', help=''),
            make_option('-u', '--user', dest='user', help=''),
            make_option('-p', '--pass', dest='password', help=''),]

        return options


class Post(XMLRPCAction):
    """An Action for creating new posts on a remote server."""

    name = "post"
    summary = "Create a new blog post on a remote server."
    usage = "%prog post [site]"

    def getPostText(self):
        """Return the text to be posted to the remote server."""
        fd, tfile = tempfile.mkstemp(prefix='clog')
        initial_content = post_template.strip()
        os.write(fd, initial_content)
        os.close(fd)

        os.system("%s %s"%(self.options.editor, tfile))
        content = open(tfile).read()

        if content != initial_content:
            return content
        else:
            return None

    def getOptions(self):
        """See `Action`."""
        options = super(Post, self).getOptions()
        options.append(
            make_option('-e', '--use-editor', dest='editor',
                default=os.environ.get("EDITOR", 'vim'),
                help=''))

        return options

    def run(self):
        """Create a new post on a remote server."""
        post_text = self.getPostText()
        if post_text is None:
            print "Post aborted."
            return

        post = BlogPost.fromText(post_text)
        post_id = self.server_proxy.newPost(post.toMetaWeblogStruct(), False)

        print post_id

register_action('post', Post)


class EditPost(XMLRPCAction):
    """An action to edit an existing post on a remote server."""

    name = "edit"
    summary = "Edit a post on a remote server."""

    def run(self):
        """Edit a Post on a remote server."""

register_action('edit', EditPost)


